package com.lzmispt.data.resource.service.collectiontool.impl;

package .tmp;

import com.alibaba.fastjson.JSON;
import com.alibaba.fastjson.TypeReference;
import com.fasterxml.jackson.core.type.TypeReference;
import com.fasterxml.jackson.databind.ObjectMapper;
import com.lzmispt.data.center.entity.dto.standard.codetable.MetadataCommonCodeItemQueryDTO;
import com.lzmispt.data.center.entity.enums.StatusCodeEnum;
import com.lzmispt.data.center.entity.vo.standard.codetable.DataRangeConfigEntity;
import com.lzmispt.data.center.entity.vo.standard.codetable.DataRangeValidateEntity;
import com.lzmispt.data.center.entity.vo.standard.codetable.MetadataCommonCodeItemVO;
import com.lzmispt.data.center.entity.vo.standard.codetable.MetadataCommonCodeVO;
import com.lzmispt.data.resource.client.DataStandardClient;
import com.lzmispt.data.resource.entity.dto.resanddata.ResAndDataFileFieldDTO;
import com.lzmispt.data.resource.entity.po.file.FileInfoFieldPO;
import com.lzmispt.data.resource.entity.po.metadata.MetadataEntityInfoItemPO;
import com.lzmispt.dcbase.dcconst.DCConst;
import com.lzmispt.framework.entity.entity.ResultEntity;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Comparator;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Objects;
import java.util.Set;
import java.util.function.Function;
import java.util.stream.Collectors;
import java.util.stream.Stream;
import jodd.util.StringUtil;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.util.CollectionUtils;

@Service
public class CollectionToolHelper {
  private static DataStandardClient DATA_STANDARD_CLIENT;
  
  @Autowired
  public void setDataStandardClient(DataStandardClient dataStandardClient) {
    DATA_STANDARD_CLIENT = dataStandardClient;
  }
  
  public static ResultEntity checkMetadataEntityDataRange(List<MetadataEntityInfoItemPO> metadataEntityInfoItemList, Map data, String entityName, String platCode, String tokenId, Boolean valueIsCodeName) throws Exception {
    List<DataRangeValidateEntity> fields = convertInfoItemToDataRangeValidateEntityList(metadataEntityInfoItemList, data);
    return checkDataRange(fields, entityName, platCode, tokenId, valueIsCodeName);
  }
  
  public static ResultEntity checkFileFieldTableDataRange(List<ResAndDataFileFieldDTO> fileInfoFields, Map data, String entityName, String platCode, String tokenId) {
    List<DataRangeValidateEntity> fields = convertFileFieldToDataRangeValidateEntityList(fileInfoFields, data);
    return checkDataRange(fields, entityName, platCode, tokenId, Boolean.valueOf(false));
  }
  
  public static ResultEntity batchCheckMetadataEntityDataRange(List<MetadataEntityInfoItemPO> metadataEntityInfoItemList, List<Map<String, Object>> dataList, String entityName, String platCode, String tokenId, Boolean valueIsCodeName) throws Exception {
    for (Map<String, Object> data : dataList) {
      ResultEntity resultEntity = checkMetadataEntityDataRange(metadataEntityInfoItemList, data, entityName, platCode, tokenId, valueIsCodeName);
      if (resultEntity.getCode().intValue() != 0)
        return resultEntity; 
    } 
    return ResultEntity.successMessage(null);
  }
  
  public static ResultEntity batchCheckFileFieldTableDataRange(List<ResAndDataFileFieldDTO> fileInfoFields, List<Map<String, Object>> dataList, String entityName, String platCode, String tokenId) throws Exception {
    for (Map<String, Object> data : dataList) {
      ResultEntity resultEntity = checkFileFieldTableDataRange(fileInfoFields, data, entityName, platCode, tokenId);
      if (resultEntity.getCode().intValue() != 0)
        return resultEntity; 
    } 
    return ResultEntity.successMessage(null);
  }
  
  public static ResultEntity checkDataRange(List<DataRangeValidateEntity> fields, List<DataRangeValidateEntity> allFields, String entityName, String platCode, String tokenId, Boolean valueIsCodeName) {
    if (fields == null || fields.isEmpty())
      return ResultEntity.successMessage(null); 
    if (allFields == null)
      allFields = fields; 
    Map<Long, DataRangeValidateEntity> fieldMap = (Map<Long, DataRangeValidateEntity>)allFields.stream().collect(Collectors.toMap(DataRangeValidateEntity::getFieldId, Function.identity()));
    Map<String, Object> resultMap = new HashMap<>();
    for (DataRangeValidateEntity field : fields) {
      if (field.getCodeTableId() == null || field.getValue() == null)
        continue; 
      MetadataCommonCodeVO codeTable = (MetadataCommonCodeVO)DATA_STANDARD_CLIENT.getMetadataCommonCodeById(field.getCodeTableId(), platCode, tokenId).getObj();
      if (codeTable == null)
        continue; 
      List<MetadataCommonCodeItemVO> commonCodeItems = getCommonCodeItems(codeTable.getCodeTableId(), platCode, tokenId);
      if (commonCodeItems == null || commonCodeItems.isEmpty())
        return errorResult(StatusCodeEnum.DATA_RESOURCE_CODE_ITEM_NOT_SET, String.format("实体【%s】字段【%s】的值域验证不通过，配置的代码表不存在或未设置代码项", new Object[] { entityName, field.getFieldName() })); 
      if (field.getValue().isEmpty())
        return errorResult(StatusCodeEnum.DATA_RESOURCE_VALUE_DOES_NOT_MATCH_VALUE_DOMAIN_CONFIGURATION, String.format("实体【%s】字段【%s】的值域验证不通过，字段值与值域配置不匹配", new Object[] { entityName, field.getFieldName() })); 
      Map<Integer, List<MetadataCommonCodeItemVO>> metadataCommonCodeItemListMap = new HashMap<>();
      if (field.getConfigs() == null && !StringUtil.isEmpty(field.getValue())) {
        List<MetadataCommonCodeItemVO> metadataCommonCodeItemVOList = (List<MetadataCommonCodeItemVO>)commonCodeItems.stream().filter(f -> f.getCodeName().equals(field.getValue())).collect(Collectors.toList());
        metadataCommonCodeItemListMap.put(Integer.valueOf(0), metadataCommonCodeItemVOList);
        if (!CollectionUtils.isEmpty(metadataCommonCodeItemVOList))
          resultMap.put(field.getFieldName(), ((MetadataCommonCodeItemVO)metadataCommonCodeItemVOList.get(0)).getCodeValue()); 
        continue;
      } 
      ResultEntity<Map<String, Object>> resultEntity = checkMultistageDataRange(entityName, valueIsCodeName, fieldMap, resultMap, field, codeTable, commonCodeItems, metadataCommonCodeItemListMap);
      if (!resultEntity.getCode().equals(DCConst.FALSE))
        return resultEntity; 
      resultMap = (Map<String, Object>)resultEntity.getObj();
    } 
    return ResultEntity.successMessage(resultMap);
  }
  
  private static ResultEntity<Map<String, Object>> checkMultistageDataRange(String entityName, Boolean valueIsCodeName, Map<Long, DataRangeValidateEntity> fieldMap, Map<String, Object> resultMap, DataRangeValidateEntity field, MetadataCommonCodeVO codeTable, List<MetadataCommonCodeItemVO> commonCodeItems, Map<Integer, List<MetadataCommonCodeItemVO>> metadataCommonCodeItemListMap) {
    List<DataRangeConfigEntity> configs = field.getConfigs();
    List<MetadataCommonCodeItemVO> matchedItems = new ArrayList<>(commonCodeItems);
    List<DataRangeConfigEntity> notSelects = (List<DataRangeConfigEntity>)configs.stream().filter(item -> !Boolean.TRUE.equals(item.getSelected())).sorted(Comparator.comparingInt(DataRangeConfigEntity::getLevel)).collect(Collectors.toList());
    for (DataRangeConfigEntity notSelect : notSelects) {
      if (notSelect.getLinkFieldId() == null)
        continue; 
      DataRangeValidateEntity relateField = fieldMap.get(notSelect.getLinkFieldId());
      if (relateField == null || relateField.getValue() == null)
        continue; 
      if (relateField.getValue().isEmpty())
        return errorResult(StatusCodeEnum.DATA_RESOURCE_THE_ASSOCIATED_FIELD_IS_NOT_IN_THE_CORRESPONDING_LEVEL, 
            String.format("实体【%s】字段【%s】的值域验证不通过，关联字段【%s】的值不在对应层级代码项中", new Object[] { entityName, field.getFieldName(), relateField.getFieldName() })); 
      List<MetadataCommonCodeItemVO> relateCodeItems = new ArrayList<>();
      String[] arrayOfString = relateField.getValue().replace("[", "").replace("]", "").split(codeTable.getDataRangeSplitSymbol());
      for (String value : arrayOfString) {
        MetadataCommonCodeItemVO relateCodeItem = matchedItems.stream().filter(item -> (Objects.equals(item.getLevel(), notSelect.getLevel()) && valueIsCodeName.booleanValue()) ? value.equalsIgnoreCase(item.getCodeName()) : value.equalsIgnoreCase(item.getCodeValue())).findFirst().orElse(null);
        if (relateCodeItem == null)
          return errorResult(StatusCodeEnum.DATA_RESOURCE_THE_ASSOCIATED_FIELD_IS_NOT_IN_THE_CORRESPONDING_LEVEL, String.format("实体【%s】字段【%s】的值域验证不通过，关联字段【%s】的值不在对应层级代码项中", new Object[] { entityName, field.getFieldName(), relateField.getFieldName() })); 
        relateCodeItems.add(relateCodeItem);
      } 
      Set<Long> relateCodeItemIds = (Set<Long>)relateCodeItems.stream().map(MetadataCommonCodeItemVO::getCodeItemId).collect(Collectors.toSet());
      matchedItems.removeIf(item -> (Objects.equals(item.getLevel(), notSelect.getLevel()) && !relateCodeItemIds.contains(item.getCodeItemId())));
      removeNotMatchedCodeItemUpward(matchedItems, relateCodeItems, notSelect.getLevel().intValue());
      removeNotMatchedCodeItemDownward(matchedItems, relateCodeItems, notSelect.getLevel().intValue());
    } 
    Set<Integer> selectedLevel = getSelectedLevel(configs);
    Integer maxLevel = selectedLevel.stream().max(Comparator.comparingInt(item -> item.intValue())).orElse(null);
    Integer minLevel = selectedLevel.stream().min(Comparator.comparingInt(item -> item.intValue())).orElse(null);
    String[] values = field.getValue().split(codeTable.getDataRangeSplitSymbol());
    List<String> resultList = new ArrayList<>(values.length);
    for (int i = 0; i < values.length; i++) {
      String value = values[i];
      List<MetadataCommonCodeItemVO> metadataCommonCodeItemVOList = new ArrayList<>();
      Map<Integer, String> map = new HashMap<>();
      boolean result = recursiveCheckDataRangeIsMatch(matchedItems, null, selectedLevel, minLevel, maxLevel, value.toLowerCase(), valueIsCodeName, map, metadataCommonCodeItemVOList);
      if (!result)
        return errorResult(StatusCodeEnum.DATA_RESOURCE_VALUE_DOES_NOT_MATCH_VALUE_DOMAIN_CONFIGURATION, String.format("实体【%s】字段【%s】的值域验证不通过，字段值与值域配置不匹配", new Object[] { entityName, field.getFieldName() })); 
      StringBuilder sb = new StringBuilder();
      map.entrySet().stream().sorted(Comparator.comparing(Map.Entry::getKey)).forEach(entry -> sb.append((String)entry.getValue()));
      resultList.add(sb.toString());
      metadataCommonCodeItemListMap.put(Integer.valueOf(i), metadataCommonCodeItemVOList);
    } 
    resultMap.put(field.getFieldName(), joinStrings(resultList, codeTable.getDataRangeSplitSymbol()));
    resultMap.put("matchedItems", matchedItems);
    resultMap.put("metadataCommonCodeItemListMap", metadataCommonCodeItemListMap);
    return ResultEntity.successMessage(resultMap);
  }
  
  public static ResultEntity checkDataRange(List<DataRangeValidateEntity> fields, String entityName, String platCode, String tokenId, Boolean valueIsCodeName) {
    return checkDataRange(fields, null, entityName, platCode, tokenId, valueIsCodeName);
  }
  
  public static List<MetadataCommonCodeItemVO> getCommonCodeItems(Long codeTableId, String platCode, String tokenId) {
    MetadataCommonCodeItemQueryDTO metadataCommonCodeItemQueryDTO = new MetadataCommonCodeItemQueryDTO();
    metadataCommonCodeItemQueryDTO.setPageIndex(Integer.valueOf(-1));
    metadataCommonCodeItemQueryDTO.setCodeTableId(codeTableId);
    metadataCommonCodeItemQueryDTO.setIsQueryAll(DCConst.TRUE);
    return (List<MetadataCommonCodeItemVO>)DATA_STANDARD_CLIENT.getCommonCodeItemsByCodeTableId(metadataCommonCodeItemQueryDTO, platCode, tokenId).getObj();
  }
  
  private static Set<Integer> getSelectedLevel(List<DataRangeConfigEntity> configs) {
    return (Set<Integer>)configs.stream()
      .filter(item -> Boolean.TRUE.equals(item.getSelected()))
      .map(DataRangeConfigEntity::getLevel)
      .collect(Collectors.toSet());
  }
  
  public static Map<String, Object> checkAndGetDataRange(MetadataEntityInfoItemPO metadataEntityInfoItemPO, List<MetadataEntityInfoItemPO> allMetadataEntityInfoItem, String entityName, String platCode, String tokenId, Boolean valueIsCodeName, Map<String, Object> rowData) {
    DataRangeValidateEntity field = convertToDataRangeValidateEntity(metadataEntityInfoItemPO, rowData);
    List<DataRangeValidateEntity> allFields = convertInfoItemToDataRangeValidateEntityList(allMetadataEntityInfoItem, rowData);
    return checkAndGetDataRangeInternal(field, allFields, entityName, platCode, tokenId, valueIsCodeName);
  }
  
  public static Map<String, Object> checkAndGetDataRange(List<ResAndDataFileFieldDTO> allFileInfoFieldList, FileInfoFieldPO curFileInfoField, String entityName, String platCode, String tokenId, Boolean valueIsCodeName, Map<String, Object> rowData) {
    DataRangeValidateEntity field = convertToDataRangeValidateEntity(curFileInfoField, rowData);
    List<DataRangeValidateEntity> allFields = convertFileFieldToDataRangeValidateEntityList(allFileInfoFieldList, rowData);
    return checkAndGetDataRangeInternal(field, allFields, entityName, platCode, tokenId, valueIsCodeName);
  }
  
  private static Map<String, Object> checkAndGetDataRangeInternal(DataRangeValidateEntity field, List<DataRangeValidateEntity> allFields, String entityName, String platCode, String tokenId, Boolean valueIsCodeName) {
    ResultEntity result = checkDataRange(Collections.singletonList(field), allFields, entityName, platCode, tokenId, valueIsCodeName);
    if (result != null && result.getCode().intValue() == 0 && result.getObj() != null) {
      Map<String, Object> resultMap = (Map<String, Object>)result.getObj();
      return resultMap;
    } 
    return null;
  }
  
  private static ResultEntity errorResult(StatusCodeEnum statusCode, String message) {
    return ResultEntity.errorMessage(Integer.valueOf(statusCode.getCode()), message);
  }
  
  public static String joinStrings(List<String> list, String delimiter) {
    StringBuilder sb = new StringBuilder();
    for (int i = 0; i < list.size(); i++) {
      sb.append(list.get(i));
      if (i < list.size() - 1)
        sb.append(delimiter); 
    } 
    return sb.toString();
  }
  
  private static void removeNotMatchedCodeItemUpward(List<MetadataCommonCodeItemVO> matchedItems, List<MetadataCommonCodeItemVO> relateCodeItems, int currentLevel) {
    if (CollectionUtils.isEmpty(matchedItems) || CollectionUtils.isEmpty(relateCodeItems))
      return; 
    int parentLevel = currentLevel - 1;
    List<Long> parentIds = (List<Long>)relateCodeItems.stream().map(MetadataCommonCodeItemVO::getCode_parent_id).filter(Objects::nonNull).collect(Collectors.toList());
    List<MetadataCommonCodeItemVO> parentItems = (List<MetadataCommonCodeItemVO>)matchedItems.stream().filter(item -> (parentIds.contains(item.getCodeItemId()) && Objects.equals(item.getLevel(), Integer.valueOf(parentLevel)))).collect(Collectors.toList());
    matchedItems.removeIf(item -> (Objects.equals(item.getLevel(), Integer.valueOf(parentLevel)) && !parentIds.contains(item.getCodeItemId())));
    removeNotMatchedCodeItemUpward(matchedItems, parentItems, parentLevel);
  }
  
  public static boolean recursiveCheckDataRangeIsMatch(List<MetadataCommonCodeItemVO> matchedItems, Long parentId, Set<Integer> selectedLevel, Integer currentLevel, Integer maxLevel, String value, Boolean valueIsCodeName, Map<Integer, String> map, List<MetadataCommonCodeItemVO> selectItems) {
    if (StringUtil.isEmpty(value))
      return false; 
    Stream<MetadataCommonCodeItemVO> stream = matchedItems.stream();
    if (parentId != null)
      stream = stream.filter(item -> Objects.equals(parentId, item.getCode_parent_id())); 
    List<MetadataCommonCodeItemVO> currentLevelItems = (List<MetadataCommonCodeItemVO>)stream.filter(item -> Objects.equals(item.getLevel(), currentLevel)).collect(Collectors.toList());
    for (MetadataCommonCodeItemVO currentLevelItem : currentLevelItems) {
      if (!selectedLevel.contains(currentLevel)) {
        boolean matched = recursiveCheckDataRangeIsMatch(matchedItems, currentLevelItem.getCodeItemId(), selectedLevel, Integer.valueOf(currentLevel.intValue() + 1), maxLevel, value, valueIsCodeName, map, selectItems);
        if (matched)
          return true; 
        continue;
      } 
      if (!maxLevel.equals(currentLevel)) {
        String str = currentLevelItem.getCodeValue().toLowerCase();
        if (valueIsCodeName.booleanValue())
          str = currentLevelItem.getCodeName().toLowerCase(); 
        if (value.startsWith(str)) {
          if (StringUtil.isEmpty(StringUtil.replaceFirst(value, str, ""))) {
            map.put(currentLevelItem.getLevel(), currentLevelItem.getCodeValue());
            selectItems.add(currentLevelItem);
            return true;
          } 
          boolean matched = recursiveCheckDataRangeIsMatch(matchedItems, currentLevelItem.getCodeItemId(), selectedLevel, Integer.valueOf(currentLevel.intValue() + 1), maxLevel, StringUtil.replaceFirst(value, str, ""), valueIsCodeName, map, selectItems);
          if (matched) {
            map.put(currentLevelItem.getLevel(), currentLevelItem.getCodeValue());
            selectItems.add(currentLevelItem);
            return true;
          } 
        } 
        continue;
      } 
      String itemValue = currentLevelItem.getCodeValue().toLowerCase();
      if (valueIsCodeName.booleanValue())
        itemValue = currentLevelItem.getCodeName().toLowerCase(); 
      if (value.equalsIgnoreCase(itemValue.toLowerCase())) {
        map.put(currentLevelItem.getLevel(), currentLevelItem.getCodeValue());
        selectItems.add(currentLevelItem);
        return true;
      } 
    } 
    return false;
  }
  
  private static void removeNotMatchedCodeItemDownward(List<MetadataCommonCodeItemVO> matchedItems, List<MetadataCommonCodeItemVO> relateCodeItems, int currentLevel) {
    if (CollectionUtils.isEmpty(matchedItems) || CollectionUtils.isEmpty(relateCodeItems))
      return; 
    int childLevel = currentLevel + 1;
    List<Long> currentIds = (List<Long>)relateCodeItems.stream().map(MetadataCommonCodeItemVO::getCodeItemId).collect(Collectors.toList());
    List<MetadataCommonCodeItemVO> childItems = (List<MetadataCommonCodeItemVO>)matchedItems.stream().filter(item -> (currentIds.contains(item.getCode_parent_id()) && Objects.equals(item.getLevel(), Integer.valueOf(childLevel)))).collect(Collectors.toList());
    List<Long> childIds = (List<Long>)childItems.stream().map(MetadataCommonCodeItemVO::getCodeItemId).collect(Collectors.toList());
    matchedItems.removeIf(item -> (Objects.equals(item.getLevel(), Integer.valueOf(childLevel)) && !childIds.contains(item.getCodeItemId())));
    removeNotMatchedCodeItemDownward(matchedItems, childItems, childLevel);
  }
  
  public static DataRangeValidateEntity convertToDataRangeValidateEntity(MetadataEntityInfoItemPO metadataEntityInfoItemPO, Map<String, Object> data) {
    DataRangeValidateEntity dataRangeValidateEntity = new DataRangeValidateEntity();
    dataRangeValidateEntity.setFieldId(metadataEntityInfoItemPO.getInfoItemId());
    dataRangeValidateEntity.setFieldName(metadataEntityInfoItemPO.getDataItemName());
    dataRangeValidateEntity.setValue(getValue(data, metadataEntityInfoItemPO.getDataItemName()));
    if (metadataEntityInfoItemPO.getDataRange() != null) {
      if (metadataEntityInfoItemPO.getData_range_config() != null) {
        ObjectMapper objectMapper = new ObjectMapper();
        try {
          List<DataRangeConfigEntity> dataRangeList = objectMapper.<List<DataRangeConfigEntity>>readValue(metadataEntityInfoItemPO.getData_range_config(), (TypeReference<List<DataRangeConfigEntity>>)new Object());
          dataRangeValidateEntity.setConfigs(dataRangeList);
        } catch (Exception e) {
          throw new RuntimeException("解析数据范围配置时出错", e);
        } 
      } 
      dataRangeValidateEntity.setCodeTableId(metadataEntityInfoItemPO.getDataRange());
    } 
    return dataRangeValidateEntity;
  }
  
  private static List<DataRangeValidateEntity> convertInfoItemToDataRangeValidateEntityList(List<MetadataEntityInfoItemPO> metadataEntityInfoItemList, Map<String, Object> data) {
    List<DataRangeValidateEntity> dataRangeValidateEntityList = new ArrayList<>();
    for (MetadataEntityInfoItemPO metadataEntityInfoItemPO : metadataEntityInfoItemList) {
      DataRangeValidateEntity dataRangeValidateEntity = convertToDataRangeValidateEntity(metadataEntityInfoItemPO, data);
      dataRangeValidateEntityList.add(dataRangeValidateEntity);
    } 
    return dataRangeValidateEntityList;
  }
  
  private static DataRangeValidateEntity convertToDataRangeValidateEntity(FileInfoFieldPO fileInfoFieldPO, Map<String, Object> data) {
    DataRangeValidateEntity dataRangeValidateEntity = new DataRangeValidateEntity();
    dataRangeValidateEntity.setFieldId(fileInfoFieldPO.getFieldId());
    dataRangeValidateEntity.setFieldName(fileInfoFieldPO.getFieldName());
    dataRangeValidateEntity.setValue(getValue(data, fileInfoFieldPO.getFieldName()));
    if (fileInfoFieldPO.getDataRange() != null) {
      if (fileInfoFieldPO.getDataRangeConfig() != null) {
        List<DataRangeConfigEntity> dataRangeList = (List<DataRangeConfigEntity>)JSON.parseObject(fileInfoFieldPO.getDataRangeConfig(), (TypeReference)new Object(), new com.alibaba.fastjson.parser.Feature[0]);
        dataRangeValidateEntity.setConfigs(dataRangeList);
      } 
      dataRangeValidateEntity.setCodeTableId(fileInfoFieldPO.getDataRange());
    } 
    return dataRangeValidateEntity;
  }
  
  private static List<DataRangeValidateEntity> convertFileFieldToDataRangeValidateEntityList(List<ResAndDataFileFieldDTO> fileInfoFieldList, Map<String, Object> data) {
    List<DataRangeValidateEntity> dataRangeValidateEntityList = new ArrayList<>();
    for (FileInfoFieldPO fileInfoFieldPO : fileInfoFieldList) {
      DataRangeValidateEntity dataRangeValidateEntity = convertToDataRangeValidateEntity(fileInfoFieldPO, data);
      dataRangeValidateEntityList.add(dataRangeValidateEntity);
    } 
    return dataRangeValidateEntityList;
  }
  
  public static String getValue(Map<String, Object> data, String fieldName) {
    for (Map.Entry<String, Object> entry : data.entrySet()) {
      String currentKey = entry.getKey();
      if (currentKey.equalsIgnoreCase(fieldName)) {
        Object value = entry.getValue();
        if (value != null) {
          if (value instanceof Object[] || value instanceof ArrayList) {
            StringBuilder ret = new StringBuilder();
            List<Object> list = (List<Object>)value;
            for (Object o : list) {
              if (o instanceof Object[] || o instanceof ArrayList) {
                List<String> childValue = (List<String>)o;
                if (!StringUtil.isEmpty(ret.toString()) && !CollectionUtils.isEmpty(childValue))
                  ret.append(","); 
                for (String s : childValue)
                  ret.append(s); 
              } 
            } 
            return ret.toString();
          } 
          return value.toString();
        } 
        return null;
      } 
    } 
    return null;
  }
}
